package com.qmth.eds.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.eds.contant.SystemConstant;
import com.qmth.eds.entity.SysUser;
import com.qmth.eds.entity.SysUserRole;
import com.qmth.eds.enums.ExceptionResultEnum;
import com.qmth.eds.mapper.SysUserMapper;
import com.qmth.eds.service.SysUserRoleService;
import com.qmth.eds.service.SysUserService;
import com.qmth.eds.util.Base64Util;
import com.qmth.eds.util.ServletUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2022-05-14
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements SysUserService {

    @Resource
    private SysUserRoleService sysUserRoleService;

    @Override
    public IPage<SysUser> list(String loginName, Boolean enable, String realName, Integer pageNumber, Integer pageSize) {
        Page<SysUser> page = new Page<>(pageNumber, pageSize);
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        if (StringUtils.isNotBlank(loginName)) {
            queryWrapper.lambda().like(SysUser::getLoginName, loginName);
        }
        if (StringUtils.isNotBlank(realName)) {
            queryWrapper.lambda().like(SysUser::getRealName, realName);
        }
        if (enable != null) {
            queryWrapper.lambda().eq(SysUser::getEnable, enable);
        }
        Page<SysUser> sysUserPage = this.baseMapper.selectPage(page, queryWrapper);
        for (SysUser record : sysUserPage.getRecords()) {
            byte[] bytePassword = Base64Util.decode(record.getPassword());
            record.setPassword(new String(bytePassword));

            QueryWrapper<SysUserRole> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.lambda().eq(SysUserRole::getUserId, record.getId());
            SysUserRole sysUserRole = sysUserRoleService.getOne(queryWrapper1);
            record.setRoleId(sysUserRole != null ? sysUserRole.getRoleId() : null);
        }
        return sysUserPage;
    }

    @Transactional
    @Override
    public boolean saveUser(SysUser sysUser) {
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        if (sysUser.getId() == null) {
            SysUser user = this.baseMapper.findByMobileNumber(sysUser.getMobileNumber());
            if (user != null) {
                throw ExceptionResultEnum.ERROR.exception("手机号已注册");
            }
            sysUser.setId(SystemConstant.getDbUuid());
            sysUser.setCreateId(requestUser.getCreateId());
            sysUser.setCreateTime(System.currentTimeMillis());
            sysUser.setPassword(Base64Util.encode(sysUser.getPassword().getBytes()));
            this.baseMapper.insert(sysUser);
        } else {
            sysUser.setPassword(Base64Util.encode(sysUser.getPassword().getBytes()));
            sysUser.setUpdateId(requestUser.getUpdateId());
            sysUser.setUpdateTime(System.currentTimeMillis());
            this.baseMapper.updateById(sysUser);
        }
        // 保存角色
        sysUserRoleService.deleteByUserId(sysUser.getId());
        SysUserRole sysUserRole = new SysUserRole();
        sysUserRole.setId(SystemConstant.getDbUuid());
        sysUserRole.setUserId(sysUser.getId());
        sysUserRole.setRoleId(sysUser.getRoleId());
        return sysUserRoleService.save(sysUserRole);
    }

    @Override
    public boolean enable(SysUser user) {
        UpdateWrapper<SysUser> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(SysUser::getEnable, user.getEnable()).eq(SysUser::getId, user.getId());
        return this.update(updateWrapper);
    }

    @Override
    public boolean resetPassword(Long id) {
        String md5Password = Base64Util.encode(SystemConstant.DEFAULT_PASSWORD.getBytes());
        SysUser user = this.getById(id);
        user.setPassword(md5Password);
        return this.updateById(user);
    }
}
